/*
  emwave2d.cpp

  General purpose Electromagnetic Wave 2D Simulation 

  Copyright (c) 2007 
  
  I Wayan Sudiarta
  Dept. of Physics, Dalhousie University
  Halifax, Nova Scotia, CANADA B3H 3J5

  This program is distributed under GNU Public License
  
  NOTES:
  Some of the codes are adapted from:
  + numerical recipe in c (codes for memory allocation)
  + an example in gtk documentation (for gtkRGB)

  CHANGES:

  DATE dd/mm/yy          CHANGES
  ----------------------------------------------
  02/08/2006	add visualization using GTK
  19/01/2007    fix readbmp()
  06/02/2007    change painting pixel, now with gtkRGB       
  	
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "interface.h"
#include "support.h"
#define NR_END 1
#define FREE_ARG char*
#define  PI (4.0*atan(1.0))
#define  OMEGA (2.0*PI) 

void updatePsi(void);
void initialize(void);
gint Repaint(gpointer data);
gboolean expose_event (GtkWidget *widget, GdkEventExpose *event,gpointer user_data); 
void readbmp(char *filename, int *data);
void savebmp(char *filename1, char *filename2);

int *bmpdata;
void nrerror(char error_text[]);
float *f1d(int nl, int nh);
float **f2d(int nrl, int nrh, int ncl, int nch);
void free_f1d(float *v, int nl, int nh);
void free_f2d(float **m, int nrl, int nrh, int ncl, int nch);

//global variables

    int choice, width, thickness, wavetype, px, py;
    double vscale, vmax, refindex;
    long int isave;
    char saveFilename[40];

   int i,j,n,n2, imed,NX,NY,NT,NMED,NPML,NCB;
   int ia, ib, ja, jb, ic, jc;
   int isrc,jsrc;
   int tmpcolor;
   
   float rn,M, cabc, SIGMA_MAX, SPEED_OF_LIGHT; 
   float lambda, delt, dels, l, ra, rb, r,s,btemp, etemp, temp;
   float *epsr, *mur, *sigma, *cea, *ceb, *ch, *mr,*mi;
   float *pxa, *pya, *qxa, *qya, *rxa, *rya;
   float *pxb, *pyb, *qxb, *qyb, *rxb, *ryb;

   float ** media;
   float **ex, **ey;
   float **dx, **dy;
   float **hz, **bz;
   
   float *hiz,*eix, *eiy; /* incident fields */
   
   FILE  * outdata, * indata;
   char inputFilename[40], refIndexFilename[40];
   
static GdkPixmap *pixmap = NULL;

#define IMAGE_WIDTH	400
#define IMAGE_HEIGHT	400
#define NRGB (IMAGE_WIDTH * IMAGE_HEIGHT * 3)

guchar rgbbuf[NRGB];


int
main (int argc, char *argv[])
{
  GtkWidget *window;
  
  GtkWidget *drawing_area;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

// check if the command is correctly executed.
   if(argc<3){ 
      printf("Please type:\n %s reftypefile  refindexfile\n",argv[0]); 
      exit(1);
   }

  strcpy(inputFilename,argv[1]);
  strcpy(refIndexFilename,argv[2]);

  gtk_set_locale ();
  gtk_init (&argc, &argv);
  gdk_rgb_init();

  add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  window = create_window ();

  drawing_area = lookup_widget(GTK_WIDGET(window), "drawing_area");

  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose-event",
                      GTK_SIGNAL_FUNC (expose_event), NULL);

  gtk_widget_show (drawing_area);

  gtk_widget_show (window);

  vmax = 100; 
  width = 50;
  vscale = 2.0;
  thickness = 10;
  px = 10;
  py = 200;
  isave = 0;
  bmpdata = (int *) malloc((size_t)400*400*sizeof(int));

  readbmp(inputFilename,bmpdata);
  
  initialize();
  choice = 0;
  wavetype = 1;
  
  gtk_timeout_add(300,Repaint,(gpointer) drawing_area);


  gtk_main ();
  return 0;
}


gint Repaint(gpointer data)
{
   int x,y;
   guchar *pos;
   guchar red, green, blue; 
 
   int i1,j1, i2, j2;
   double den;
   time_t now;
   
   GtkWidget * drawing_area = (GtkWidget *) data;
   GdkDrawable *drawable;
   GdkRectangle update_rect;

   // Play 
   if(choice == 1){
      updatePsi();
   }

   // Restart
   if(choice == 2){

       for(i=0;i<=NX;i++){
       for(j=0;j<=NY;j++){
          hz[i][j] = 0;
	  ex[i][j] = 0;
	  ey[i][j] = 0;  
          bz[i][j] = 0;
	  dx[i][j] = 0;
	  dy[i][j] = 0;  
       }}
       
       for(j=0;j<=NY;j++){
	  eix[j] = 0;
	  eiy[j] = 0;  
          hiz[j] = 0;
       }

       for(i=0;i<=NMED;i++){
          epsr[i] = mr[i]*mr[i]-mi[i]*mi[i];   
          sigma[i] = 4*PI*mi[i]*mr[i]/epsr[i]; 
          mur[i] = 1.0;
       }
  
       for(i=0;i<=NMED;i++){
         temp = 0.5*delt*sigma[i];
         cea[i] = (1.0-temp)/(1.0+temp);
         ceb[i] = (delt/dels)/(epsr[i]*(1.0+temp));
         ch[i] = delt/(mur[i]*dels);
       }

       
       choice = 0;
       n = 0;
   }
   
   // Save
   if(choice == 3){
       isave++;
       sprintf(saveFilename,"Save-Hz-%ld.dat",isave);
      // SAVING and PRINTING 
      if((outdata = fopen(saveFilename,"w"))==NULL){
         fprintf(stderr," Cannot Open File \n");
         exit(3);
      }
      for(i=ia;i<=ib;i++){
      for(j=ja;j<=jb;j++){
	    fprintf(outdata,"%f ",hz[i][j]);
         }
         fprintf(outdata,"\n");
      } 
      fclose(outdata);
   
      sprintf(saveFilename,"Save-BMP-%ld.bmp",isave);
      savebmp(saveFilename,inputFilename);   
       choice = 0;
       
   }

   if(choice == 4){
      
      choice = 2;
    
   } 

   pos = rgbbuf;
   
   for(i=ia;i<ia+400;i++){
   for(j=ja;j<ja+400;j++){
      imed = media[i][j];      
      tmpcolor = 128 + (100*hz[i][j]);
      if(tmpcolor < 0) tmpcolor = 0;
      if(tmpcolor > 255) tmpcolor = 255;

      switch(imed){
        case 0:
	   red = 0.1*tmpcolor;
           green = 0.8*tmpcolor;
           blue = 0.8*tmpcolor;
           break;   	 
         case 1:
	   red = tmpcolor;
           green = 0.1*tmpcolor;
           blue = 0.1*tmpcolor;
           break;   	 
         case 2:
	   red = 0.1*tmpcolor;
           green = tmpcolor;
           blue = 0.1*tmpcolor;
           break;
	 case 3:  	 
	   red = 0.1*tmpcolor;
           green = 0.1*tmpcolor;
           blue = tmpcolor;
           break;   	 
         case -1:
           red = 0;
           green = 0;
           blue = 0;
      }
      if(i>=20 && i<=22){
       if(j>=20 && j<= (20+(int)(vscale/dels))){
          red = 255;
	  green = 255;
	  blue = 255;
	}
      }

      if(wavetype==2) if(j>=ja+px-3 && j<=ja+px+3){
       if(i<=ia+py+3 && i>= ia+py-3){
          red = 255;
	  green = 255;
	  blue = 255;
	}
      }
      
      *pos++ = red;
      *pos++ = green;
      *pos++ = blue;
   }}
   
   //Update drawing_area
   update_rect.x = 0;
   update_rect.y = 0;
   update_rect.width = drawing_area->allocation.width;
   update_rect.height = drawing_area->allocation.height;
   gtk_widget_draw(drawing_area, &update_rect);

   return TRUE;
}


void initialize(void)
{
   FILE *infile;
   float temp1, temp2;
   char stemp[80];
  
   /* INPUT PARAMETERS */
   dels = 0.03;
   delt = dels/2.0; 
   cabc = ((delt-dels)/(delt+dels));
   NMED = 10;  /* number of media excluding the host medium */ 
   NPML = 11; 
   NCB = 2;
   NX = 404 + 2*NPML;
   NY = 404 + 2*NPML;
   M = 3.0;
   SIGMA_MAX = 50.0;
   
   ia = NPML+NCB+1;
   ib = NX - NPML - NCB;
   ja = NPML+NCB+1;
   jb = NY - NPML - NCB;
  
   ic = NX/2;
   jc = NY/2;
    
   /* ALLOCATE MEMORY and INITIALIZE TO ZERO */

   /* MEDIA */
   media = f2d(0,NX,0,NY);
   
   cea = f1d(0,NMED);  ceb = f1d(0,NMED);
   ch = f1d(0,NMED); sigma = f1d(0,NMED);
   epsr = f1d(0,NMED); mur = f1d(0,NMED);
   mr = f1d(0,NMED); mi = f1d(0,NMED);

   /* UPML */
   pxa = f1d(0,NX); pya = f1d(0,NY);
   qxa = f1d(0,NX); qya = f1d(0,NY);
   rxa = f1d(0,NX); rya = f1d(0,NY);
   pxb = f1d(0,NX); pyb = f1d(0,NY);
   qxb = f1d(0,NX); qyb = f1d(0,NY);
   rxb = f1d(0,NX); ryb = f1d(0,NY);
  
   /* FIELDS */
   ex = f2d(0,NX,0,NY); ey = f2d(0,NX,0,NY);
   hz = f2d(0,NX,0,NY); dx = f2d(0,NX,0,NY);
   dy = f2d(0,NX,0,NY); bz = f2d(0,NX,0,NY);

   eix = f1d(0,NY); hiz = f1d(0,NY); eiy = f1d(0,NY);
  
   /* REF INDEX */
   mr[0] = 1.0; mi[0] = 0.0;
   // Read from File refindex.dat
   infile = fopen(refIndexFilename,"r");
   fscanf(infile,"%s  %f  %f ", stemp, &temp1, &temp2);
   mr[1] = temp1; mi[1] = temp2;
   fscanf(infile,"%s  %f  %f ", stemp, &temp1, &temp2);
   mr[2] = temp1; mi[2] = temp2;
   fscanf(infile,"%s  %f  %f ", stemp, &temp1, &temp2);
   mr[3] = temp1; mi[3] = temp2;
   fclose(infile);
   
   for(i=0;i<=NMED;i++){
      epsr[i] = mr[i]*mr[i]-mi[i]*mi[i];   
      sigma[i] = 4*PI*mi[i]*mr[i]/epsr[i]; 
      mur[i] = 1.0;
   }
  
   for(i=0;i<=NMED;i++){
      temp = 0.5*delt*sigma[i];
      cea[i] = (1.0-temp)/(1.0+temp);
      ceb[i] = (delt/dels)/(epsr[i]*(1.0+temp));
      ch[i] = delt/(mur[i]*dels);
   }

/* UPML MEDIA */

   l = NPML;

  /* INTERIOR - I initialize all to 1.0 */
   for(i=0;i<=NX;i++){
      pxa[i] = qxa[i] = rxa[i] = 1.0E0;
      pxb[i] = qxb[i] = rxb[i] = 1.0E0;
   }

   /* PML - change only for the PML region */
   for(i=1;i<=NPML;i++){
      /* half */
      rb = (l-(float)i)/l;
      ra = rb + 1.0/l;
      s = SIGMA_MAX*l*(pow(ra,M+1.0)-pow(rb,M+1.0))/(M+1.0);
      temp = 0.5*delt*s;
      pxb[i-1] = pxb[NX-i] = 1.0 - temp;
      qxb[i-1] = qxb[NX-i] = 1.0 + temp;
      rxb[i-1] = rxb[NX-i] = 1.0/qxb[i-1];
      
      /* interger */
      if(i!=NPML){
         rb = (l - (float)i-0.5)/l;
         ra = rb + 1.0/l;
         s = SIGMA_MAX*NPML*(pow(ra,M+1.0)-pow(rb,M+1.0))/(M+1.0);
      }
      else{
         ra = 0.5/l;
         s = SIGMA_MAX*l*(pow(ra,M+1.0))/(M+1.0);
      } 

      temp = 0.5*delt*s;
      pxa[i] = pxa[NX-i] = 1.0 - temp;
      qxa[i] = qxa[NX-i] = 1.0 + temp;
      rxa[i] = rxa[NX-i] = 1.0/qxa[i];

   }
   
   /* interior */
   for(j=0;j<=NY;j++){
      pya[j] = qya[j] = rya[j] = 1.0E0;
      pyb[j] = qyb[j] = ryb[j] = 1.0E0;
   } 

   for(j=1;j<=NPML;j++){
      /* half */
      l = NPML;
      rb = (l - (float)j)/l;
      ra = rb+ 1.0/l;
      s = SIGMA_MAX*l*(pow(ra,M+1.0)-pow(rb,M+1.0))/(M+1.0);
      temp = 0.5*delt*s;
      pyb[NY-j] = pyb[j-1] = 1.0 - temp;
      qyb[NY-j] = qyb[j-1] = 1.0 + temp;
      ryb[NY-j] = ryb[j-1] = 1.0/qyb[j-1];
            
      /* interger */
      if(j!=NPML){
         rb = (l - (float)j-0.5)/l;
         ra = rb + 1.0/l;
         s = SIGMA_MAX*l*(pow(ra,M+1.0)-pow(rb,M+1.0))/(M+1.0);
      }
      else{
         ra = 0.5/l;
         s = SIGMA_MAX*l*(pow(ra,M+1.0))/(M+1.0);
      } 
      temp = 0.5*delt*s;
      pya[NY-j] = pya[j] = 1.0 - temp;
      qya[NY-j] = qya[j] = 1.0 + temp;
      rya[NY-j] = rya[j] = 1.0/qya[j];

   }

   /* MEDIA INDEX or OPEN MEDIA FILE */
   

   for(i=0;i<=NX;i++){
   for(j=0;j<=NY;j++){
	   media[i][j]=0;
   }}
   
   n2 =0;
   for(i=ia;i<ia+400;i++){
   for(j=ja;j<ja+400;j++){
       media[i][j]= bmpdata[n2];
       n2++;
   }} 
    
   n = 0;
}


void updatePsi(void)
{

/* ITERATIONS */
for(n2=0;n2<10;n2++){
   n++;
   rn = n;

/* UPDATE INCIDENT FIELD */
   /* plane wave */
   if(wavetype==1){
   hiz[0] = sin(OMEGA*rn*delt/vscale);
      temp = hiz[NY-1];
      for(j=1;j<=NY-1;j++){
	hiz[j]= hiz[j] + ch[0]*(eix[j+1] - eix[j]);
      } /* j*/
      /* absorbing boundary - one way */
      hiz[NY]=temp + cabc*(hiz[NY-1]-hiz[NY]);
   }
/* UPDATE MAGNETIC FIELDS */
   for(i=0;i<=NX-1;i++){
   for(j=0;j<=NY-1;j++){
      //imed = 0;
      imed = media[i][j];
      if(imed != -1){
         btemp = bz[i][j];
         bz[i][j]= pxb[i]*rxb[i]*bz[i][j]
            + rxb[i]*ch[imed]*(ex[i][j+1] - ex[i][j]- ey[i+1][j] + ey[i][j]);
         hz[i][j]= ryb[j]*(pyb[j]*hz[i][j] + (bz[i][j] - btemp));
      }
      else{
         hz[i][j] = 0;  
      }
      
   }} /* ij */

   if(wavetype == 2){
      //point source
      hz[py+ic-200][px+jc-200] = hz[py+ic-200][px+jc-200] + 50*sin(OMEGA*rn*delt/vscale);
   }
   

/* UPDATE - CONNECTING BOUNDARY */
   if(wavetype == 1){
      /* plane x = npml +1 */

      i = NPML + NCB-1;
      for(j=NPML+NCB;j<=NY-NPML-NCB-1;j++){
         hz[i][j] = hz[i][j] + ch[0]*eiy[j];
      }
   
      /* plane x = NX - NPML +1  */
      i = NX-NPML-NCB;
      for(j=NPML+NCB;j<=NY-NPML-NCB-1;j++){
         hz[i][j] = hz[i][j] - ch[0]*eiy[j];
      }

      /* plane y = npml +1 */

      j = NPML + NCB-1;
      for(i=NPML+NCB;i<=NX-NPML-NCB-1;i++){
         hz[i][j] = hz[i][j] - ch[0]*eix[j];
      }

      /* plane y = NY - npml - 1 */

      j = NY - NPML - NCB;
      for(i=NPML+NCB;i<=NX-NPML-NCB-1;i++){
         hz[i][j] = hz[i][j] + ch[0]*eix[j];
      }
   
    
      /* UPDATE INCIDENT FIELD */
      for(j=1;j<=NY;j++){
         eix[j]= cea[0]*eix[j]+ ceb[0]*(hiz[j] - hiz[j-1]);
      } /* ijk */
   } 
/* ELECTRIC FIELDS */
   for(i=0;i<=NX;i++){
   for(j=0;j<=NY;j++){
      imed = media[i][j];
      if(imed == -1) imed = 0;
      if(j!=0 && j!=NY){
	 etemp = dx[i][j];
  	 dx[i][j]= pya[j]*rya[j]*cea[imed]*dx[i][j]
            + rya[j]*ceb[imed]*(hz[i][j] - hz[i][j-1]);
         ex[i][j]= ex[i][j] + (qxb[i]*dx[i][j] - pxb[i]*etemp);
      }
      else {
	/* PEC BOUNDDARY */
         ex[i][j] = 0.0;
      }
      if(i!=0 && i!=NX){
	 etemp = dy[i][j];
	 dy[i][j]= pxa[i]*rxa[i]*cea[imed]*dy[i][j]
             + rxa[i]*ceb[imed]*(hz[i-1][j]- hz[i][j]);
	 ey[i][j]= ey[i][j] + (qyb[j]*dy[i][j]- pyb[j]*etemp);
      }
      else {
	/* PEC BOUNDDARY */
         ey[i][j] = 0.0;
      }
   }} /* ij */

   if(wavetype == 1){
      /* UPDATE - CONNECTING BOUNDARY */

      /* plane x = npml + NCB */

      i = NPML + NCB;
      for(j=NPML+NCB;j<=NY-NPML-NCB-1;j++){
         ey[i][j] = ey[i][j] + ceb[0]*hiz[j];
      }
   
      /* plane x = NX - NPML + NCB  */
      i = NX-NPML-NCB;
      for(j=NPML+NCB;j<=NY-NPML-NCB-1;j++){
         ey[i][j] = ey[i][j] - ceb[0]*hiz[j];
      }

      /* plane y = npml +1 */

      j = NPML + NCB;
      for(i=NPML+NCB;i<NX-NPML-NCB-1;i++){
         ex[i][j] = ex[i][j] - ceb[0]*hiz[j-1];
      }

      /* plane y = NY - npml - 1 */

      j = NY - NPML - NCB;
      for(i=NPML+NCB;i<NX-NPML-NCB-1;i++){
         ex[i][j] = ex[i][j] + ceb[0]*hiz[j];
      }
   }
}/* n iterations */

}



float *f1d(int nl, int nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
	int i;
	float *v;

	v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
	if (!v) nrerror("allocation failure in f1d()");

	/* initialize to zero */
	for(i=nl;i<=nh;i++) v[i] = 0.0;

	/* return the vector */
	return v-nl+NR_END;
}


float **f2d(int nrl, int nrh, int ncl, int nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
	int i, j, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	float **m;

	/* allocate pointers to rows */
	m=(float **) malloc((size_t)((nrow+NR_END)*sizeof(float*)));
	if (!m) nrerror("allocation failure 1 in f2d()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(float *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
	if (!m[nrl]) nrerror("allocation failure 2 in f2d()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* initialize to zero */
	for(i=nrl;i<=nrh;i++){
	   for(j=ncl;j<=nch;j++){
		 m[i][j] = 0.0;
	   }
	}

	/* return pointer to array of pointers to rows */
	return m;
}

void free_f2d(float **m, int nrl, int nrh, int ncl, int nch)
/* free a float matrix allocated by matrix() */
{
	free((FREE_ARG) (m[nrl]+ncl-NR_END));
	free((FREE_ARG) (m+nrl-NR_END));
}

void free_f1d(float *v, int nl, int nh)
/* free a float vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"FDTD run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}


gboolean expose_event (GtkWidget *widget,
		 GdkEventExpose *event,
		 gpointer user_data)
{
  gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
		      0, 0, IMAGE_WIDTH, IMAGE_HEIGHT,
		      GDK_RGB_DITHER_MAX, rgbbuf, IMAGE_WIDTH * 3);
}

// Read data from BMP file
void readbmp(char *filename, int *data)
{
    FILE *infile;   
    char *tempData; 
    long int i, j, n, n2, ired, igreen, iblue;
    long int w, h; 
    long int offset, padWidth, byteWidth, dataSize;
    
    //open file in binary mode
    infile=fopen(filename,"rb");
    if(infile==NULL) { printf("Input BMP file %s can not be found \n",filename);
        fclose(infile);
	exit(1);
    }

    // get width
    fseek(infile,18,SEEK_SET);
    fread(&w,sizeof(int),(size_t)1,infile);
    
    // get height
    fseek(infile,22,SEEK_SET);
    fread(&h,sizeof(int),(size_t)1,infile);

    //witdh in bytes
    byteWidth=(long int)(w*3.0);
    //printf("\n %d %d \n",w,h);
    
    //compute offset
    offset = 0;
    if(byteWidth%4 != 0) offset = (4 - byteWidth%4);

    //calc data size
    dataSize=(w*3+offset)*h;

    //allocate memory
    tempData=(char *) malloc(dataSize*sizeof(char));
    if(tempData==NULL) {
        printf("Problem in memory allocation !\n");
    }

    //read image
    fseek(infile,54,SEEK_SET);
    fread(tempData,sizeof(char),dataSize,infile);

    fclose(infile);

    // Get data
    n = 0; n2=0;
    for(i=h-1; i>0;i--){ 
       for(j=0; j<w;j++){
           n2 = (i*w + j);
	  iblue = (unsigned char) tempData[n];
	  n++;
	  igreen = (unsigned char) tempData[n];
	  n++;
	  ired = (unsigned char) tempData[n];
          n++;
	        
          if(ired == 255 && igreen == 255 && iblue == 255){// white 
	      data[n2]=0;
	  }
          if(ired == 0 && igreen == 0 && iblue == 0){// black 
	      data[n2]=-1;
	  }

          if(ired == 255 && igreen == 0 && iblue == 0){// red 
	      data[n2]=1;
	  }

          if(ired == 0 && igreen == 255 && iblue == 0){// green 
	      data[n2]=2;
	  }

          if(ired == 0 && igreen == 0 && iblue == 255){// blue 
	      data[n2]=3;
	  }

       }
       n = n + offset;
    }
}


// Read data from BMP file
void savebmp(char *savefile, char *inputfile)
{
    FILE *infile, *outfile;   
    char *tempData, *offsetData,*tempHeader; 
    long int i, j, n, n2, ired, igreen, iblue;
    long int w, h; 
    long int offset, padWidth, byteWidth, dataSize;
    
    //open file in binary mode
    infile=fopen(inputfile,"rb");
    if(infile==NULL) { printf("Input BMP file %s can not be found \n",inputfile);
        fclose(infile);
	exit(1);
    }

    outfile=fopen(savefile,"wb");
    if(infile==NULL) { printf("Input BMP file %s can not be found \n",savefile);
        fclose(infile);
	exit(1);
    }

    // get width
    fseek(infile,18,SEEK_SET);
    fread(&w,sizeof(int),(size_t)1,infile);
    
    // get height
    fseek(infile,22,SEEK_SET);
    fread(&h,sizeof(int),(size_t)1,infile);

    //witdh in bytes
    byteWidth=(long int)(w*3.0);
    //printf("\n %d %d \n",w,h);
    
    //compute offset
    offset = 0;
    if(byteWidth%4 != 0) offset = (4 - byteWidth%4);
    if(offset > 0) offsetData=(char *) malloc(offset*sizeof(char));

    //calc data size
    dataSize=(w*3+offset)*h;
    tempData=(char *) malloc(dataSize*sizeof(char));
    //allocate memory
    tempHeader=(char *) malloc(54*sizeof(char));
    if(tempData==NULL) {
        printf("Problem in memory allocation !\n");
    }

    //read image
    fseek(infile,0,SEEK_SET);
    fread(tempHeader,sizeof(char),54,infile);
    fwrite(tempHeader,sizeof(char),54,outfile);
    
    fclose(infile);

    // Save datas
    n=0;
    for(i=h-1; i>0;i--){ 
       for(j=0; j<w;j++){
	  n2 = (i*w + j)*3;
	  tempData[n+2] = (char) rgbbuf[n2];
          tempData[n+1] = (char) rgbbuf[n2+1];
          tempData[n] = (char) rgbbuf[n2+2];
	  n = n + 3;
       }
       if(offset >0) n = n + offset;
    }
    fwrite(tempData,sizeof(char),dataSize,outfile);
    
    fclose(outfile);
}

